klout-ios-sdk
=============

Simple SDK for KloutPass


### Using starter example ###
* ```git clone git@github.com:klout/klout-ios-sdk.git```
* ```git clone git@github.com:klout/klout-ios-starter.git```
* ```open klout-ios-starter/klout-ios-starter.xcodeproj```
* You may have to build first before running.


### Creating your own ###

* Create Xcode project
* From Finder, drag klout-ios-sdk.xcodeproj into your Xcode project
* In Build Phases/Target Dependencies, add klout-ios-sdk
* In Build Phases/Link Binary With Libraries, add klout-ios-sdk
* In Build Settings/Header Search Paths, add $BUILT_PRODUCTS_DIR
* If you would like to use the KloutPassButton and would like a cooler looking cancel button for the webview, drag Resources from klout-ios-sdk into your app.


### SDK ###

#### KloutPassAdapter ####

``` objective-c
-(id) initWithSuperView:(UIView*)superview 
           withDelegate:(id <KloutPassAdapterDelegate>)delegate;
```
Initialize the adapter with a superview that will encompass the oauth webview, a delegate that will handle callbacks (see next section), your api key, secret, and redirect.

``` objective-c
-(void) login;
```
Begin the login process. Will show webview.

``` objective-c
-(BOOL) isLoggedIn;
```
Check if the adapter is in the logged in state.

``` objective-c
-(void) logout;
```
Logout the current session.

``` objective-c
- (void) requestUser;
```
When logged in, will request a user payload for the current logged in user.


#### KloutPassAdapterDelegate ####
``` objective-c
- (void) completedLogin;
```
Finished login.

``` objective-c
- (void) completedUserFetch:(NSString*)userPayload;
```
Finished user fetch for current user.

``` objective-c
- (void) error:(KloutPassError*)theError;
```
An error occured. See below for KloutPassError documentation.

#### KloutPassError ####
``` objective-c
- (NSString*) errorMessage;
```
Get the error message

``` objective-c
- (NSString*) errorCode;
```
Get the error code. Code can take on the following values:
* error.connection - Connection error
* error.login.not - Not logged in (when making fetchUserPayload call)
* error.login.already - Already logged in (when making login call)
* error.user-request.already - Already making user request
* error.token-request.already - Already making token request
* error.unknown - Unknown error

``` objective-c
- (NSString*) apiKey;
```
Return the api key for your app

``` objective-c
- (NSString*) apiSecret;
```
Return the secret for your app

``` objective-c
- (NSString*) redirect;
```
Return the redirect for your app


#### KloutPassButton ####
This is a login button you can just drop in Interface Builder. The asset size is 274x49. It would behoove you to adhere to that ratio.

* Use a typical UIButton, but change the class to KloutPassButton
* If you're using InterfaceBuilder, call ```[KloutPassButton version];``` somewhere in your app
* Set the delegate for the button, either via the outlet in InterfaceBuilder or in code.