//
//  KloutPassAdapter.m
//  klout-ios-sdk
//
//  Created by Jieren Chen on 9/4/12.
//  Copyright (c) 2012 Klout. All rights reserved.
//

#import "KloutPassAdapter.h"

@implementation KloutPassAdapter


@synthesize currentState, apiKey, apiSecret, apiRedirect, currentKloutId, theView, theDelegate, theConnection, responseData, accessToken, activityView, cancelButton;


- (id) initWithSuperView:(UIView*)superview withDelegate:(id <KloutPassAdapterDelegate>)delegate {    
    self = [super init];
    if (!self) {        
        return nil;
    }
    else {
        
        self.apiKey = [delegate apiKey];
        self.apiSecret = [delegate apiSecret];
        self.apiRedirect = [delegate redirect];
        self.theDelegate = delegate;
        self.responseData = [NSMutableData data];
        
        NSString* loginUrl = [NSString stringWithFormat:@"https://%@/%@/oauth/?apiKey=%@&redirect=%@", BASE_URL,VERSION_NUM,self.apiKey,self.apiRedirect];

        //Create sub-views
        //Activity indicator
        self.activityView = [[[UIView alloc] initWithFrame:[superview bounds]] autorelease];
        UIActivityIndicatorView* activityIndicator = [[[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(0, 0, 50, 50)] autorelease];
        activityIndicator.center = self.activityView.center;
        [self.activityView addSubview:activityIndicator];
        [activityIndicator setColor:[UIColor orangeColor]];
        [activityIndicator startAnimating];
        
        //cancel button
        self.cancelButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        UIImage* cancelImage = [UIImage imageNamed:@"close-button"];
        if (cancelImage) {
            self.cancelButton = [UIButton buttonWithType:UIButtonTypeCustom];
            self.cancelButton.frame = CGRectMake(0, 0, 30, 30);
            [self.cancelButton setImage:cancelImage forState:UIControlStateNormal];
        }
        else {
            self.cancelButton.frame = CGRectMake(0, 0, 30, 30);
            [self.cancelButton setTitle:@"X" forState:UIControlStateNormal];
            [self.cancelButton.titleLabel setTextAlignment:UITextAlignmentCenter];            
        }
        
        NSLog(@"%@", loginUrl);
        self.cancelButton.center = CGPointMake(superview.frame.size.width - 20, 20);
        [self.cancelButton addTarget:self action:@selector(cancel:) forControlEvents:UIControlEventTouchUpInside];
        
        //web view
        self.theView = [[[KloutPassWebView alloc] initWithFrame:[superview bounds] withURL:loginUrl withDelegate:self withRedirect:self.apiRedirect] autorelease];
        
        [self.theView reset];
        
        //Add views
        [superview addSubview:self.theView];
        [superview addSubview:self.activityView]; 
        [superview addSubview:self.cancelButton];

        self.activityView.hidden = YES;
        self.theView.hidden = YES;
        self.cancelButton.hidden = YES;

        self.currentState = LOGGED_OUT;
        return self;
    }
}


- (void) login {
    if (![self isLoggedIn]) {
        NSURL* theURL = self.theView.request.mainDocumentURL;
        
        if (!([[theURL host] isEqualToString:BASE_URL] &&
              [[theURL path] isEqualToString:[NSString stringWithFormat:@"/%@/oauth/",VERSION_NUM]])) {
            NSLog(@"%@",theURL);
            [self.theView reset];
            self.theView.hidden = YES;
            self.activityView.hidden = NO;
        }
        else {
            self.theView.hidden = NO;
            self.activityView.hidden = YES;
        }
        self.cancelButton.hidden = NO;
        self.currentState = LOGGING_IN;
    }
    else if (self.currentState == REQUESTING_TOKEN) {        
        self.theView.hidden = YES;
        self.activityView.hidden = YES;
        self.cancelButton.hidden = YES;
        KloutPassError* theError = [[[KloutPassError alloc] initWithType:ALREADY_LOGGED_IN] autorelease];
        [self.theDelegate error:theError];
    }
    else {
        //error
        self.theView.hidden = YES;
        self.activityView.hidden = YES;
        self.cancelButton.hidden = YES;
        KloutPassError* theError = [[[KloutPassError alloc] initWithType:ALREADY_LOGGED_IN] autorelease];
        [self.theDelegate error:theError];
    }
}

- (BOOL) isLoggedIn {
    
    return self.currentState == LOGGED_IN || self.currentState == REQUESTING_USER;
}

- (void) logout {
    self.currentState = LOGGED_OUT;
    self.accessToken = nil;
    [self.theConnection cancel];
}

- (IBAction)cancel:(id)sender {
    
    [self.theView stopLoading];
    self.theView.hidden = YES;
    self.activityView.hidden = YES;
    self.cancelButton.hidden = YES;
    self.currentState = LOGGED_OUT;
}

- (void) requestUser {
    NSLog(@"Requesting user");
    if (self.accessToken != nil && self.currentKloutId && self.currentState != REQUESTING_USER) {
        
        NSString* urlString = [NSString stringWithFormat:@"https://%@/%@/user.oauth/%@",BASE_URL,VERSION_NUM,self.currentKloutId];
        //request user.json
        NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlString]
                                                               cachePolicy:NSURLRequestReloadIgnoringLocalAndRemoteCacheData timeoutInterval:10];
        
        [request setValue:[NSString stringWithFormat:@"Bearer %@", self.accessToken] forHTTPHeaderField:@"Authorization"];
        
        [request setHTTPMethod:@"GET"];
        [self.responseData setLength:0];
        self.currentState = REQUESTING_USER;
        self.theConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
        
        if (!self.theConnection) {
            KloutPassError* theError = [[[KloutPassError alloc] initWithType:CONNECTION_FAILURE] autorelease];
            [self.theDelegate error:theError];
        }
    }
    else if (self.currentState == REQUESTING_USER) {
        KloutPassError* theError = [[[KloutPassError alloc] initWithType:ALREADY_REQUESTING_USER] autorelease];
        [self.theDelegate error:theError];
    }
    else {
        KloutPassError* theError = [[[KloutPassError alloc] initWithType:ALREADY_LOGGED_IN] autorelease];
        [self.theDelegate error:theError];        
    }
}

#pragma web view delegate
- (void)webViewDidStartLoad:(UIWebView *)webView
{
    NSLog(@"start load");
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    if (self.currentState == LOGGING_IN){
        self.activityView.hidden = NO;
//        self.theView.hidden = YES;
    }
    
}

- (void)webViewDidFinishLoad:(UIWebView *)webView
{
    NSLog(@"finish load");
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
    if (self.currentState == LOGGING_IN){
        self.activityView.hidden = YES;
        self.theView.hidden = NO;
        self.cancelButton.hidden = NO;
    }
}

#pragma private methods

- (void) requestAccessTokenWithAuthCode:(NSString*)authCode withUser:(NSString*) kloutId {
    
    if (self.accessToken == nil && self.currentState == LOGGING_IN) {
        NSString* apiSig = [self generateApiSignature];        
        
        NSString* urlString = [NSString stringWithFormat:@"https://%@/%@/oauth/token?authCode=%@&user=%@&apiKey=%@&apiSig=%@&redirect=%@",BASE_URL,VERSION_NUM,authCode,kloutId,self.apiKey,apiSig,self.apiRedirect];
        
        
        NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlString]
                                                               cachePolicy:NSURLRequestReloadIgnoringLocalAndRemoteCacheData timeoutInterval:10];
        [request setHTTPMethod:@"GET"];
        [self.responseData setLength:0];
        self.currentState = REQUESTING_TOKEN;
        self.theConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
        
        if (!self.theConnection) {
            KloutPassError* theError = [[[KloutPassError alloc] initWithType:CONNECTION_FAILURE] autorelease];
            [self.theDelegate error:theError];
        }
        
    }
    else {
        KloutPassError* theError = [[[KloutPassError alloc] initWithType:ALREADY_LOGGED_IN] autorelease];
        [self.theDelegate error:theError];
    }
}

- (void) completedWithAuthCode:(NSString *)authCode withKloutId:(NSString *)kloutId {
    
    NSLog(@"authCode %@ %@",authCode, kloutId);
    //set access token with a request
    
    self.currentKloutId = kloutId;
    self.theView.hidden = YES;
    self.cancelButton.hidden = YES;
    [self requestAccessTokenWithAuthCode:authCode withUser:kloutId];
}

- (NSString*)generateApiSignature {
    //this can be problematic
    NSString* ts = [[NSNumber numberWithLongLong:[[NSDate date] timeIntervalSince1970] / 10] stringValue];
    
    NSLog(@"tiemstamp: %@",ts);
    NSString* digest = [[self.apiKey stringByAppendingString:self.apiSecret] stringByAppendingString:ts];
    
    const char *cStr = [digest UTF8String];
    unsigned char result[16];
    CC_MD5( cStr, strlen(cStr), result);
    
    return [NSString stringWithFormat:
            @"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
            result[0], result[1], result[2], result[3], 
            result[4], result[5], result[6], result[7],
            result[8], result[9], result[10], result[11],
            result[12], result[13], result[14], result[15]
            ];
}

#pragma urlrequest

- (void) connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    KloutPassError* theError = [[[KloutPassError alloc] initWithType:CONNECTION_FAILURE] autorelease];
    [self.theDelegate error:theError];
}

- (void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    [self.responseData appendData:data];    
}

- (void) connectionDidFinishLoading:(NSURLConnection*) connection
{
    NSString* parsedData = [[[NSString alloc] initWithData:self.responseData encoding:NSUTF8StringEncoding] autorelease];
    if (self.currentState == REQUESTING_TOKEN && connection == self.theConnection) {
        self.accessToken = parsedData;
        NSLog(@"access token %@",self.accessToken);
        self.currentState = LOGGED_IN;
        self.activityView.hidden = YES;
        [self.theDelegate completedLogin];
    }
    if (self.currentState == REQUESTING_USER && connection == self.theConnection) {
        NSLog(@"user payload %@",parsedData);
        self.currentState = LOGGED_IN;
        [self.theDelegate completedUserFetch:parsedData];
    }
}

- (void) dealloc {
    self.accessToken = nil;
    self.apiKey = nil;
    self.apiSecret = nil;
    self.apiRedirect = nil;
    self.currentKloutId = nil;
    self.theView = nil;
    self.activityView = nil;
    self.cancelButton = nil;
    self.theDelegate = nil;
    self.theConnection = nil;
    self.responseData = nil;
    
    [super dealloc];
}
@end
